;***********************************************************************
;
;	Microchip 16-bit Embedded Control Design Contest
;
;	Entry # MT2268
;
;	Spa Pump Controller - Spa Interface Processor
;
;***********************************************************************
;
;	Systemwide Definitions
;
;***********************************************************************

#include	"Macros.inc"


; Pump drive unit communication definitions

CMD_set_spd	equ	6		; Set speed command

FREQ_MIN	equ	20		; Minimum pump operating frequency (Hz)
FREQ_MAX	equ	60		; Maximum pump operating frequency (Hz)
FREQ_INIT	equ	30		; Initial pump operating frequency (Hz)

RESP_TIME	equ	100		; Maximum command response time (msec)

BAUD		equ	9600		; Serial baud rate


; Keypad scanning definitions

KEY_TIME	equ	10		; Key scan interval (msec)
KEY_DEB		equ	10		; Key debounce time (# of intervals)
KEY_HOLD	equ	150		; Key hold detection delay (# of intervals)
KEY_RPT		equ	10		; Key repeat delay (# of intervals)

KEYOUT_TIME	equ	10		; Key output interval (x 20 msec)


K_HOLD		equ	7		; Key held down flag bit
K_RPT		equ	6		; Key repeat flag bit
K_REL		equ	5		; Key released flag bit

KV_hold		equ	(1<<K_HOLD)	; Key held down value
KV_rpt		equ	KV_hold+(1<<K_RPT) ; Key repeat value
KV_rel		equ	(1<<K_REL)	; Key released value

KV_none		equ	0		; Spa control key values
KV_up		equ	1		; These match the actual values
KV_down		equ	2		; send by the spa control keypad
KV_filter	equ	3
KV_econ		equ	4
KV_light	equ	5
KV_pump2	equ	6
KV_pump1	equ	7
KV_pump3	equ	8


; LCD display icons (flag bits in LCD_icons)

ICON_pump1	equ	0		; Pump 1 running


; Fault flags (flag bits in Sys_fault)

FAULT_resp	equ	0		; Serial commm response timeout
FAULT_lcd	equ	1		; LCD frame timeout


; I/O pin definitions

POT_PORT	equ	PORTC		; Digital pot pins
POT_CS		equ	4
POT_UD		equ	5

ADC_pump1	equ	4<<2		; Pump 1 sensor ADC channel (currently unused)
ADC_pump2	equ	5<<2		; Pump 2 sensor ADC channel (currently unused)
ADC_ozone	equ	6<<2		; Ozone sensor ADC channel (currently unused)
ADC_keypad	equ	7<<2		; Keypad ADC channel


